"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const nodegit_1 = require("nodegit");
const jiraClient_1 = require("../jiraClient");
const githubClient_1 = require("../githubClient");
const inquirer = require("inquirer");
const base_1 = require("../base");
class Pr extends base_1.default {
    constructor() {
        super(...arguments);
        this.interactive = false;
    }
    async run() {
        const { flags } = this.parse(Pr);
        this.interactive = flags["interactive"];
        // JIRA
        const jiraUser = await this.getFlagValue(flags, "jira-email");
        const jiraAccessToken = await this.getFlagValue(flags, "jira-access-token");
        const jiraHost = await this.getFlagValue(flags, "jira-host", "jira.atlassian.com");
        const jiraTicketId = await this.getFlagValue(flags, "ticket-id");
        const jiraTicket = await this.getJiraTicket(jiraAccessToken, jiraHost, jiraUser, jiraTicketId);
        // GitHub
        var baseBranch = "master";
        if (this.interactive) {
            const repo = await nodegit_1.Repository.open(".");
            const branches = (await repo.getReferences(3 /* LISTALL */))
                .filter((ref) => { return ref.isBranch(); })
                .map((ref) => { return { name: ref.shorthand() }; });
            const answer = await inquirer.prompt([{ name: "branch", message: "base-branch", type: "list", choices: branches }]);
            baseBranch = answer.branch;
        }
        else {
            baseBranch = await this.getFlagValue(flags, "base-branch", "master");
        }
        const githubAccessToken = await this.getFlagValue(flags, "github-access-token");
        const prTitle = await this.getFlagValue(flags, "pr-title", jiraTicket.fields.summary);
        const prTitleWithTicketId = this.createPRTitle(prTitle, jiraTicket);
        const prDescription = this.createPRDescription(jiraHost, jiraTicket);
        const pr = await this.makePullRequest(githubAccessToken, baseBranch, prTitleWithTicketId, prDescription);
        this.log(pr.html_url);
    }
    async getJiraTicket(accessToken, host, username, ticketId) {
        let jiraClient = new jiraClient_1.JiraClient({
            username: username,
            accessToken: accessToken,
            host: host
        });
        cli_ux_1.default.action.start("Fetching JIRA ticket");
        const jiraTicket = await jiraClient.getJiraTicket(ticketId);
        cli_ux_1.default.action.stop("done");
        return jiraTicket;
    }
    async makePullRequest(accessToken, base, title, description) {
        let githubClient = new githubClient_1.GitHubClient(accessToken);
        cli_ux_1.default.action.start("Making pull request");
        const result = await githubClient.openPullRequest({
            repo: await nodegit_1.Repository.open("."),
            title: title,
            description: description,
            base: base
        });
        cli_ux_1.default.action.stop("done");
        return result;
    }
    createPRTitle(title, jiraTicket) {
        return `[${jiraTicket.key}] ${title}`;
    }
    createPRDescription(jiraHost, jiraTicket) {
        return `https://${jiraHost}/browse/${jiraTicket.key}`;
    }
    async getFlagValue(flags, flag, defaultValue) {
        const value = flags[flag];
        if (value) {
            return value;
        }
        else if (this.interactive) {
            return await cli_ux_1.default.prompt(flag, { required: defaultValue === undefined, default: defaultValue });
        }
        else {
            return defaultValue || this.error(`Missing --${flag} flag!`);
        }
    }
}
Pr.description = "Create GitHub PRs from JIRA tickets";
Pr.flags = Object.assign({}, base_1.default.flags, { "help": command_1.flags.help({ char: "h" }), "base-branch": command_1.flags.string({ char: "b", description: "base branch for PR" }), "ticket-id": command_1.flags.string({ char: "t", description: "jira ticket ID" }), "pr-title": command_1.flags.string({ description: "custom PR title" }), "interactive": command_1.flags.boolean({ char: "i", description: "interactive mode", default: false }) });
exports.default = Pr;
